@@ -1,5 +1,5 @@
 #
-#Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009 Olivier Sessink
+#Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009, 2010, 2011 Olivier Sessink
 #All rights reserved.
 #
 #Redistribution and use in source and binary forms, with or without
@@ -65,12 +65,6 @@ def path_is_safe(path, failquiet=0):
 		if (failquiet == 0):
 			sys.stderr.write('ERROR: cannot lstat() '+path+'\n')
 		return -1
-	if (not stat.S_ISDIR(statbuf[stat.ST_MODE])):
-		if (stat.S_ISLNK(statbuf[stat.ST_MODE])):
-			sys.stderr.write('ERROR: '+path+' is a symlink, please point to the real directory\n')
-		else:
-			sys.stderr.write('ERROR: '+path+' is not a directory!\n')
-		return -2
 	if (sys.platform[-3:] == 'bsd'):
 		# on freebsd root is in group wheel
 		if (statbuf[stat.ST_UID] != 0 or statbuf[stat.ST_GID] != grp.getgrnam('wheel').gr_gid):
@@ -80,9 +74,20 @@ def path_is_safe(path, failquiet=0):
 		if (statbuf[stat.ST_UID] != 0 or statbuf[stat.ST_GID] != 0):
 			sys.stderr.write('ERROR: '+path+' is not owned by root:root!\n')
 			return -3
-	if (statbuf[stat.ST_MODE] & stat.S_IWOTH or statbuf[stat.ST_MODE] & stat.S_IWGRP):
+	if ((statbuf[stat.ST_MODE] & stat.S_IWOTH or statbuf[stat.ST_MODE] & stat.S_IWGRP)and not stat.S_ISLNK(statbuf[stat.ST_MODE])):
 		sys.stderr.write('ERROR: '+path+' is writable by group or others!')
 		return -4
+	if (not stat.S_ISDIR(statbuf[stat.ST_MODE])):
+		if (stat.S_ISLNK(statbuf[stat.ST_MODE])):
+			# Fedora has moved /sbin /lib and /bin into /usr
+			target = os.readlink(path)
+			print target, path
+			if (target[:4] != 'usr/'):
+				sys.stderr.write('ERROR: '+path+' is a symlink, please point to the real directory\n')
+				return -2
+		else:
+			sys.stderr.write('ERROR: '+path+' is not a directory!\n')
+			return -2
 	return 1
 
 def chroot_is_safe(path, failquiet=0):
@@ -97,7 +102,7 @@ def chroot_is_safe(path, failquiet=0):
 			return retval
 	npath = os.path.dirname(path)
 	while (npath != '/'):
-		print npath
+		#print npath
 		retval = path_is_safe(npath,0)
 		if (retval != 1):
 #			print 'testing path='+npath+'returned '+str(retval)
@@ -315,10 +320,12 @@ def copy_time_and_permissions(src, dst, be_verbose=0, allow_suid=0, copy_ownersh
 			mode = (mode & ~stat.S_ISUID) & ~stat.S_ISGID
 #			print 'setuid!!! after mode='+str(mode)
 #		print 'mode='+str(mode)
-	os.chmod(dst, mode)
 	os.utime(dst, (sbuf[stat.ST_ATIME], sbuf[stat.ST_MTIME]))
 	if (copy_ownership):
 		os.chown(dst, sbuf[stat.ST_UID], sbuf[stat.ST_GID])
+	#WARNING: chmod must be AFTER chown to preserve setuid/setgid bits
+	os.chmod(dst, mode)
+
 
 def OLDreturn_existing_base_directory(path):
 	"""This function tests if a directory exists, if not tries the parent etc. etc. until it finds a directory that exists"""
@@ -392,9 +399,21 @@ def OLD_create_parent_path(chroot, path, be_verbose=0, copy_permissions=1, allow
 			if (indx==-1):
 				indx=len(directory)
 
+def fix_double_slashes(instring):
+	outstring=''
+	slash=0
+	for i in instring:
+		if (slash==0 or i!= '/'):
+			outstring += i
+		if (i == '/'):
+			slash=1
+		else:
+			slash=0
+	return outstring
+
 def split_path(path):
 	ret = []
-	next=path
+	next=fix_double_slashes(os.path.normpath(path))
 	while (next != '/'):
 		ret.insert(0,os.path.basename(next))
 		next=os.path.dirname(next)
@@ -506,8 +525,8 @@ def move_dir_with_permissions_and_owner(srcdir,dstdir,be_verbose=0):
 	else:
 		print 'Not everything was copied to '+dstdir+', keeping the old directory '+srcdir
 
-def copy_with_permissions(src, dst, be_verbose=0, try_hardlink=1, retain_owner=0):
-	"""copies/links the file and the permissions, except any setuid or setgid bits"""
+def copy_with_permissions(src, dst, be_verbose=0, try_hardlink=1, allow_suid=0, retain_owner=0):
+	"""copies/links the file and the permissions (and possibly ownership and setuid/setgid bits"""
 	do_normal_copy = 1
 	if (try_hardlink==1):
 		try:
@@ -519,7 +538,7 @@ def copy_with_permissions(src, dst, be_verbose=0, try_hardlink=1, retain_owner=0
 	if (do_normal_copy == 1):
 		try:
 			shutil.copyfile(src,dst)
-			copy_time_and_permissions(src, dst, be_verbose, allow_suid=0, copy_ownership=retain_owner)
+			copy_time_and_permissions(src, dst, be_verbose, allow_suid=allow_suid, copy_ownership=retain_owner)
 		except (IOError, OSError), (errno,strerror):
 			sys.stderr.write('ERROR: copying file and permissions '+src+' to '+dst+': '+strerror+'\n')
 
@@ -563,21 +582,21 @@ def copy_device(chroot, path, be_verbose=1, retain_owner=0):
 		print 'use "mknod '+chrootpath+' mode major minor" to create the device'
 		print 'use chmod and chown to set the permissions as found by ls -l'
 
-def copy_dir_recursive(chroot,dir,force_overwrite=0, be_verbose=0, check_libs=1, try_hardlink=1, retain_owner=0, handledfiles=[]):
-	"""copies a directory and the permissions recursive, except any setuid or setgid bits"""
+def copy_dir_recursive(chroot,dir,force_overwrite=0, be_verbose=0, check_libs=1, try_hardlink=1, allow_suid=0, retain_owner=0, handledfiles=[]):
+	"""copies a directory and the permissions recursively, possibly with ownership and setuid/setgid bits"""
 	files2 = ()
 	for entry in os.listdir(dir):
 		tmp = os.path.join(dir, entry)
 		try:
 			sbuf = cachedlstat(tmp)
 			if (stat.S_ISDIR(sbuf.st_mode)):
-				create_parent_path(chroot, tmp, be_verbose=be_verbose, copy_permissions=1, allow_suid=0, copy_ownership=retain_owner)			
-				handledfiles = copy_dir_recursive(chroot,tmp,force_overwrite, be_verbose, check_libs, try_hardlink, retain_owner, handledfiles)
+				create_parent_path(chroot, tmp, be_verbose=be_verbose, copy_permissions=1, allow_suid=allow_suid, copy_ownership=retain_owner)			
+				handledfiles = copy_dir_recursive(chroot,tmp,force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 			else:
 				files2 += os.path.join(dir, entry),
 		except OSError, e:
 			sys.stderr.write('ERROR: failed to investigate source file '+tmp+': '+e.strerror+'\n')
-	handledfiles = copy_binaries_and_libs(chroot,files2,force_overwrite, be_verbose, check_libs, try_hardlink, retain_owner, handledfiles)
+	handledfiles = copy_binaries_and_libs(chroot,files2,force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 	return handledfiles 
 #	for root, dirs, files in os.walk(dir):
 #		files2 = ()
@@ -596,7 +615,7 @@ def copy_dir_recursive(chroot,dir,force_overwrite=0, be_verbose=0, check_libs=1,
 # try to lstat(/srv/jail/opt/bin/foo) and you get the result for /usr/bin/foo
 # so use resolve_realpath to find you want lstat(/srv/jail/usr/bin/foo)
 #
-def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0, check_libs=1, try_hardlink=1, retain_owner=0, try_glob_matching=0, handledfiles=[]):
+def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0, check_libs=1, try_hardlink=1, allow_suid=0, retain_owner=0, try_glob_matching=0, handledfiles=[]):
 	"""copies a list of executables and their libraries to the chroot"""
 	if (chroot[-1] == '/'):
 		chroot = chroot[:-1]
@@ -620,7 +639,7 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 				sys.stderr.write('ERROR: failed to investigate source file '+file+': '+e.strerror+'\n')
 			continue
 		# source file exists, resolve the chroot realfile
-		create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=0, copy_ownership=retain_owner)
+		create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=allow_suid, copy_ownership=retain_owner)
 		chrootrfile = resolve_realpath(os.path.normpath(chroot+'/'+file),chroot)
 		#if (rfile in handledfiles):
 		#	create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=0, copy_ownership=retain_owner)
@@ -661,7 +680,7 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 						if (be_verbose):
 							print 'Destination file '+chrootrfile+' exists'
 						continue
-			create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=0, copy_ownership=retain_owner)
+			create_parent_path(chroot,os.path.dirname(file), be_verbose, copy_permissions=1, allow_suid=allow_suid, copy_ownership=retain_owner)
 			if (stat.S_ISLNK(sb.st_mode)):
 				realfile = os.readlink(file)
 				try:
@@ -673,15 +692,15 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 				handledfiles.append(file)
 				if (realfile[0] != '/'):
 					realfile = os.path.normpath(os.path.join(os.path.dirname(file),realfile))
-				handledfiles = copy_binaries_and_libs(chroot, [realfile], force_overwrite, be_verbose, check_libs, try_hardlink, retain_owner, handledfiles)
+				handledfiles = copy_binaries_and_libs(chroot, [realfile], force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 			elif (stat.S_ISDIR(sb.st_mode)):
-				handledfiles = copy_dir_recursive(chroot,file,force_overwrite, be_verbose, check_libs, try_hardlink, retain_owner, handledfiles)
+				handledfiles = copy_dir_recursive(chroot,file,force_overwrite, be_verbose, check_libs, try_hardlink, allow_suid, retain_owner, handledfiles)
 			elif (stat.S_ISREG(sb.st_mode)):
 				if (try_hardlink):
 					print 'Trying to link '+file+' to '+chrootrfile
 				else:
 					print 'Copying '+file+' to '+chrootrfile
-				copy_with_permissions(file,chrootrfile,be_verbose, try_hardlink, retain_owner)
+				copy_with_permissions(file,chrootrfile,be_verbose, try_hardlink, allow_suid, retain_owner)
 				handledfiles.append(file)
 			elif (stat.S_ISCHR(sb.st_mode) or stat.S_ISBLK(sb.st_mode)):
 				copy_device(chroot, file, be_verbose, retain_owner)
@@ -692,7 +711,7 @@ def copy_binaries_and_libs(chroot, binarieslist, force_overwrite=0, be_verbose=0
 			mode = stat.S_IMODE(sb[stat.ST_MODE])
 			if (check_libs and (string.find(file, 'lib') != -1 or string.find(file,'.so') != -1 or (mode & (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)))):
 				libs = lddlist_libraries(file)
-				handledfiles = copy_binaries_and_libs(chroot, libs, force_overwrite, be_verbose, 0, try_hardlink, handledfiles)
+				handledfiles = copy_binaries_and_libs(chroot, libs, force_overwrite, be_verbose, 0, try_hardlink, handledfiles=handledfiles)
 	return handledfiles
 
 def config_get_option_as_list(cfgparser, sectionname, optionname):
@@ -814,3 +833,23 @@ def init_passwd_and_group(chroot,users,groups,be_verbose=0):
 			line = fd.readline()
 		fd.close()
 	fd2.close()
+
+def find_file_in_path(filename):
+	search_path = os.getenv('PATH')
+	paths = string.split(search_path, ':')
+	for path in paths:
+		joined = os.path.join(path, filename)
+		if os.path.exists(joined):
+			return os.path.abspath(joined)
+	return None
+
+def find_files_in_path(paths):
+	paths2 = []
+	for tmp in paths:
+		if (tmp[0] == '/'):
+			paths2.append(tmp)
+		else:
+			tmp2 = find_file_in_path(tmp)
+			if (tmp2):
+				paths2.append(tmp2)
+	return paths2
